//
// Created by zhangkg on 2022/8/31.
//

#ifndef SAMI_CORE_KARAOKE_EDIT_H
#define SAMI_CORE_KARAOKE_EDIT_H

#import <Foundation/Foundation.h>
#include "SAMICoreCode.h"

#ifdef __cplusplus
extern "C" {
#endif

@interface SAMICore_KaraokeEditSettingParam : NSObject
@property (nonatomic, assign) const char* _Nonnull vocal_path;
@property (nonatomic, assign) const char* _Nonnull bgm_path;
@property (nonatomic, assign) const char* _Nonnull record_result_path;
@property (nonatomic, assign) int sample_rate;
@property (nonatomic, assign) int max_block_samples;
@end

@interface SAMICore_LoudnormProperty : NSObject
@property (nonatomic, assign) float source_lufs;
@property (nonatomic, assign) float source_peak;
@property (nonatomic, assign) float target_lufs;
@end

@interface SAMICoreKaraokeEdit: NSObject

- (_Nullable instancetype)initEditGraphWithSettingParam:(SAMICore_KaraokeEditSettingParam* _Nonnull)x
                                                 result:(int* _Nonnull)result;

- (int)pullAudioData:(float* _Nonnull* _Nonnull)out_data
         num_samples:(int)num_samples;

- (void)prepare;
- (void)play;
- (void)pause;
- (void)stop;
- (void)seek:(float)time_ms;

// 获取播放轴的总时间：一般为干声文件的总时长. 需要init之后调用
-(float)getTotalDuratioinMs;

// 降噪功能相关
- (int)setDenoiseModelPath:(const char* _Nonnull)denoise_model_path; // 仅需调用一次，调用后则开启降噪
- (int)setUseDenoise:(bool)use_denoise;

// 响度均衡
- (int)setBGMLoudnormInfo:(SAMICore_LoudnormProperty* _Nonnull)bgm_loudnorm;

// 设置/切换音效
- (int)updateEffectFilePath:(const char* _Nonnull)effect_path;
- (int)setUseEffect:(bool)use_effect;

// 修音相关
- (int)setByteTunerParam;
- (int)getByteTunerStatus;
- (int)setUseByteTuner:(bool)use_byteTuner;

// 音量调整
- (int)updateVocalVolume:(float)value_db;
- (int)updateBGMVolume:(float)value_db;

// 调整人声伴奏偏移值
- (int)setVocalOffsetMs:(float)vocal_offset_ms;
@end

#ifdef __cplusplus
}   // extern "C"
#endif
#endif  //SAMI_CORE_KARAOKE_EDIT_H